/***************************************************************************
                          sankaku.h  -  description
                             -------------------
    begin                : Thu Nov 21 2002
    copyright            : (C) 2002 by Nobuhiro Kuroiwa
    email                : nobu@iastate.edu
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef SANKAKU_H
#define	SANKAKU_H

#include <iostream>
#include <cstdlib>
#include <string>

#include "SDL/SDL.h"
#include "SDL/sge.h"

#include "matrix.h"
#include "projtool.h"

//---------------------------------------------------------------------------
//	Class SankakuCallBack
//---------------------------------------------------------------------------
class SankakuCallBack
{
public:
	SankakuCallBack();
	virtual void reportBox( SDL_Rect rect, unsigned short tirg_index) = 0;
	virtual void reportPercent( int per) = 0;
	virtual ~SankakuCallBack();
};

//---------------------------------------------------------------------------
//	Class Sankaku
//---------------------------------------------------------------------------
class Sankaku
{
protected:
	SDL_Surface		*sfc_result;
	SDL_Surface		*sfc_mercator;
	SDL_Surface		*sfc_progress;
	SDL_Surface		*sfc_temp_unfold;
	SDL_Surface		*sfc_trig_array[4];
	bool			ready2play;
	bool			stop;
	unsigned int	current_trig;

	unsigned int	WIDTH_TRIANGLE;		// size of a side of triangle 
	unsigned int	HEIGHT_TRIANGLE;	// height of triangle
	double			FACTOR_H;			// = width sfc_mercator / (2PI)
	double			FACTOR_W;			// = height sfc_mercator / 2.0
	double			RESOLUTION;			// = 1/resolution
	int				min_pix;
	SankakuCallBack *pcallback;

	bool createSurface(
					SDL_Surface*	&sfc_in,
					unsigned int	width,
					unsigned int	height,
					bool			flg_clear = false
	);
	bool blitSurface(
					SDL_Surface*	&surface_in,
					NUM_TYPE		x0,
					NUM_TYPE		y0,
					NUM_TYPE		ext_width,
					NUM_TYPE		ext_height,
					SDL_Surface*	&surface_out,
					int				destx,
					SDL_Rect		*rect_copied
	);
	bool extractSurfaceFromMercator(
					Square			sq,
					SDL_Surface*	&psurface_out,
					int				destx_max,
					NUM_TYPE		destx,
					SDL_Rect		*rect_copied
	);
	unsigned int getBoxOnMercator(
					Square			&sq_in,
					Square			*psq_array_ret
	);
	SDL_Rect drawProcessingBox(
					Square &sq_in
	);
	bool extractSurfaceFromProcessing(
					SDL_Rect		&rect_in,
					Square			&sq,
					unsigned	int	index_trig
	);
	bool			unfold();
	bool			normalizeMercator();
	bool			checkProgSfcSize();
	bool			load(	string	*pstr_file_in);
	void			drawTab();
	void			drawLine4Tab( Square &sq);
	void			freeEmAllButMercator();
	void			nullEmAllButMercator();

public:     	
					Sankaku( string *pstr_file2load);
					~Sankaku();
	bool			play();
	bool			isReady()				const;
	bool			save(	string	*pstr_file_out);
	bool			initSankaku(
					unsigned int	reso,
					unsigned int	sz);
	void			stopIt();
	void			setCallBack( SankakuCallBack	*pclient);
	int				getTrigIndex()			const;
	SDL_Surface 	*getResultSurface()		const;
	SDL_Surface 	*getMercatorSurface()	const;


};

bool transformSurfaceEx(
					SDL_Surface *original,
					SDL_Surface *&newsfc,
					int			sz_w,
					int			sz_h,
					SDL_Rect	*prct_result,
					bool		exact = false
);
void freeSurface(	SDL_Surface* &sfc);

/*// For Debug
//#ifndef	_DEBUG
void printRect( char *name, SDL_Rect *rect, bool nl = true);
void printRect( char *name, SDL_Surface *sfc, bool nl = true);
void saveSurfacePieces( char *name, SDL_Surface *sfc);
int myitoa( int d, char *string, int base = 10);
//#endif*/
#endif
